from typing import Any, Optional
import logging

from django import template

from ..models import Link

logger = logging.getLogger(__name__)

register = template.Library()


@register.simple_tag
def get_wagtail_link(link_name: str) -> Optional[Link]:
    """
    Get a Link object by its name.
    """
    try:
        link = Link.objects.get_by_natural_key(link_name)
    except Link.DoesNotExist:
        logger.error("Wagtail Link with name [%s] does not exist.", link_name)
        return None
    return link


@register.simple_tag
def get_wagtail_link_url(
    link_name: str,
    localized: bool = True,
    *args: Any,
    **kwargs: Any,
) -> str:
    """
    Get a Link URL by its Link name.
    """
    link = get_wagtail_link(link_name)
    if link is None:
        return ""
    return link.get_url(localized=localized, *args, **kwargs)



from typing import Any
import logging

from django.urls import NoReverseMatch, reverse
from django.utils.translation import gettext

from .models import Link

logger = logging.getLogger(__name__)


class DefaultLinkResolver:
    def get_descr(self, link: Link) -> str:
        url = link.url
        if not url:
            url = gettext("No Link URL")
        ctx = {
            "name": link.name,
            "link_external": link.link_external,
            "link_relative": link.link_relative,
            "link_page": link.link_page,
            "django_view_name": link.django_view_name,
            "url": url,
        }
        if link.name:
            return gettext("Link[name=%(name)s]: %(url)s") % ctx
        if link.link_external:
            return gettext("Link[link_external=%(link_external)s]: %(url)s") % ctx
        if link.link_relative:
            return gettext("Link[link_relative=%(link_relative)s]: %(url)s") % ctx
        if link.link_page:
            return gettext("Link[link_page=%(link_page)s]: %(url)s") % ctx
        return gettext("Link[django_view_name=%(django_view_name)s]: %(url)s") % ctx

    def get_url(
        self,
        link: Link,
        localized: bool = True,
        *args: Any,
        **kwargs: Any,
    ) -> str:
        # 1. External Links
        if link.link_external:
            return link.link_external
        # 2. Relative Links
        if link.link_relative:
            return link.link_relative
        # 3. Wagtail Page Links
        if link.link_page:
            if localized:
                url = link.link_page.localized.url
            else:
                url = link.link_page.url
            if url is None:
                return ""
            return url
        # 4. Django View Links
        try:
            return reverse(link.django_view_name)
        except NoReverseMatch:
            logger.warning("Unable to reverse Django URL for Link[id=%s]", link.pk)
        # 5. Error Fallback
        return ""



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-05-22 15:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.core.validators


class Migration(migrations.Migration):
    dependencies = [
        ("wagtail_links", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="link",
            name="link_relative",
            field=models.CharField(
                blank=True,
                max_length=200,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Not valid relative url",
                        regex="^(?!www\\.|(?:http|ftp)s?://|[A-Za-z]:\\\\|//).*",
                    )
                ],
                verbose_name="Relative link",
            ),
        ),
    ]



from typing import Any, Protocol, Tuple
import logging

from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.db import models
from django.urls import NoReverseMatch, reverse
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _
from django_stubs_ext.db.models import TypedModelMeta
from wagtail.admin.panels import FieldPanel, PageChooserPanel
from wagtail.search import index
from wagtail.snippets.models import register_snippet

from .fields import NullSlugField

logger = logging.getLogger(__name__)


def validate_django_reverse(view_name: str) -> None:
    try:
        reverse(view_name)
    except NoReverseMatch:
        # Translators: CMS error message
        raise ValidationError(_("Invalid Django view name"))


class LinkResolver(Protocol):
    def get_descr(self, link: "Link") -> str: ...  # NOQA: E704

    def get_url(  # NOQA: E704
        self,
        link: "Link",
        localized: bool = True,
        *args: Any,
        **kwargs: Any,
    ) -> str: ...


class LinkManager(models.Manager["Link"]):
    def get_by_natural_key(self, name: str) -> "Link":
        return self.get(name=name)


@register_snippet
class Link(index.Indexed, models.Model):
    """
    A generic link that points somewhere else using various methods
    """

    title = models.CharField(
        _("Title"),
        max_length=200,
        blank=True,
        help_text=_("Description of link for use in ARIA compliance"),
    )
    name = NullSlugField(
        _("Name"),
        unique=True,
        help_text=_("Unique name for this link (for use by Django templates)."),
    )
    link_external = models.URLField(
        _("External Link"), blank=True, help_text=_("Absolute URL Link")
    )
    link_relative = models.CharField(
        _("Relative Link"),
        max_length=200,
        blank=True,
        validators=[
            RegexValidator(
                regex=r"^(?!www\.|(?:http|ftp)s?://|[A-Za-z]:\\|//).*",
                message="Not valid relative url",
            )
        ],
        help_text=_("Relative URL Link"),
    )
    link_page = models.ForeignKey(
        "wagtailcore.Page",
        verbose_name=_("Wagtail Page Link"),
        null=True,
        blank=True,
        related_name="+",
        on_delete=models.CASCADE,
        help_text=_("Wagtail Page Link"),
    )
    django_view_name = models.CharField(
        _("Django View Link"),
        max_length=255,
        blank=True,
        validators=[validate_django_reverse],
        help_text=_("Name of Django view for reverse lookup"),
    )

    objects = LinkManager()

    panels = [
        FieldPanel("title"),
        FieldPanel("name"),
        FieldPanel("link_external"),
        FieldPanel("link_relative"),
        PageChooserPanel("link_page"),
        FieldPanel("django_view_name"),
    ]

    search_fields = [
        index.RelatedFields(
            "link_page",
            [
                index.AutocompleteField("title"),
            ],
        ),
        index.AutocompleteField("title"),
    ]

    class Meta(TypedModelMeta):
        # Translators: Internal Model Name (singular)
        verbose_name = _("Link")
        # Translators: Internal Model Name (plural)
        verbose_name_plural = _("Links")

    def natural_key(self) -> Tuple[str]:
        return (self.name,)

    def __str__(self) -> str:
        return self.get_resolver().get_descr(self)

    @property
    def url(self) -> str:
        """Get URL for use in template"""
        return self.get_url()

    def clean(self) -> None:
        # Don't allow multiple link types to be used
        number_used = (
            bool(self.link_external)
            + bool(self.link_relative)
            + bool(self.link_page)
            + bool(self.django_view_name)
        )
        if number_used > 1:
            raise ValidationError(_("You may only use one link type"))
        if number_used == 0:
            raise ValidationError(_("You must use exactly one link type"))

    def get_resolver(self) -> LinkResolver:
        conf: dict[str, Any] = getattr(
            settings,
            "WAGTAIL_LINKS",
            {},
        )
        resolver_path: str = conf.get(
            "RESOLVER",
            "wagtail_links.resolver.DefaultLinkResolver",
        )
        resolver_opts: dict[str, Any] = conf.get("RESOLVER_OPTIONS", {})
        Resolver: type[LinkResolver] = import_string(resolver_path)
        return Resolver(**resolver_opts)

    def get_url(self, *args: Any, **kwargs: Any) -> str:
        resolver = self.get_resolver()
        return resolver.get_url(self, *args, **kwargs)



from collections.abc import Sequence
from typing import Any, Optional

from django.core.exceptions import ImproperlyConfigured
from django.db.models.fields import SlugField


class NullSlugField(SlugField):  # type:ignore[type-arg]
    """
    SlugField that stores '' as None and returns None as ''

    Useful when using unique=True and forms. Implies null==blank==True.
    """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        if not kwargs.get("null", True) or not kwargs.get("blank", True):
            raise ImproperlyConfigured("NullCharField implies null==blank==True")
        kwargs["null"] = kwargs["blank"] = True
        super().__init__(*args, **kwargs)

    def from_db_value(
        self,
        value: Optional[str],
        *args: Any,
        **kwargs: Any,
    ) -> Optional[str]:
        value = self.to_python(value)
        # If the value was stored as null, return empty string instead
        return value if value is not None else ""

    def get_prep_value(self, value: Optional[str]) -> Optional[str]:
        prepped = super().get_prep_value(value)
        # If the value was stored as empty string, return None instead
        return prepped if prepped != "" else None

    def deconstruct(self) -> tuple[str, str, Sequence[Any], dict[str, Any]]:
        name, path, args, kwargs = super().deconstruct()
        del kwargs["null"]
        del kwargs["blank"]
        return name, path, args, kwargs



# Generated by Django 2.1.8 on 2019-04-11 21:02

from django.db import migrations, models
import django.core.validators
import django.db.models.deletion

import wagtail_links.models


class Migration(migrations.Migration):
    dependencies = [
        ("wagtail_links", "0003_auto_20170522_2015"),
    ]

    operations = [
        migrations.AddField(
            model_name="link",
            name="name",
            field=models.SlugField(
                help_text="Unique name for this link (for use by Django templates).",
                null=True,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="link",
            name="django_view_name",
            field=models.CharField(
                blank=True,
                help_text="Name of Django view for reverse lookup",
                max_length=255,
                validators=[wagtail_links.models.validate_django_reverse],
            ),
        ),
        migrations.AlterField(
            model_name="link",
            name="link_external",
            field=models.URLField(
                blank=True, help_text="Absolute URL Link", verbose_name="External link"
            ),
        ),
        migrations.AlterField(
            model_name="link",
            name="link_page",
            field=models.ForeignKey(
                blank=True,
                help_text="Wagtail Page Link",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="wagtailcore.Page",
            ),
        ),
        migrations.AlterField(
            model_name="link",
            name="link_relative",
            field=models.CharField(
                blank=True,
                help_text="Relative URL Link",
                max_length=200,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Not valid relative url",
                        regex="^(?!www\\.|(?:http|ftp)s?://|[A-Za-z]:\\\\|//).*",
                    )
                ],
                verbose_name="Relative link",
            ),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-04 21:30

from django.db import migrations, models
import django.core.validators
import django.db.models.deletion

import wagtail_links.models


class Migration(migrations.Migration):
    replaces = [
        ("wagtail_links", "0005_auto_20191203_1908"),
        ("wagtail_links", "0006_auto_20191204_2129"),
    ]

    dependencies = [
        ("wagtail_links", "0004_auto_20190411_2102"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="link",
            options={"verbose_name": "Link", "verbose_name_plural": "Links"},
        ),
        migrations.AlterField(
            model_name="link",
            name="django_view_name",
            field=models.CharField(
                blank=True,
                help_text="Name of Django view for reverse lookup",
                max_length=255,
                validators=[wagtail_links.models.validate_django_reverse],
                verbose_name="Django View Link",
            ),
        ),
        migrations.AlterField(
            model_name="link",
            name="link_external",
            field=models.URLField(
                blank=True, help_text="Absolute URL Link", verbose_name="External Link"
            ),
        ),
        migrations.AlterField(
            model_name="link",
            name="link_page",
            field=models.ForeignKey(
                blank=True,
                help_text="Wagtail Page Link",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="wagtailcore.Page",
                verbose_name="Wagtail Page Link",
            ),
        ),
        migrations.AlterField(
            model_name="link",
            name="link_relative",
            field=models.CharField(
                blank=True,
                help_text="Relative URL Link",
                max_length=200,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Not valid relative url",
                        regex="^(?!www\\.|(?:http|ftp)s?://|[A-Za-z]:\\\\|//).*",
                    )
                ],
                verbose_name="Relative Link",
            ),
        ),
        migrations.AlterField(
            model_name="link",
            name="name",
            field=models.SlugField(
                blank=True,
                help_text="Unique name for this link (for use by Django templates).",
                null=True,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AddField(
            model_name="link",
            name="title",
            field=models.SlugField(
                blank=True,
                help_text="Description of link for use in ARIA compliance",
                verbose_name="Title",
            ),
        ),
    ]
